﻿/*
 * Copyright (c) 2023 Proton AG
 *
 * This file is part of ProtonVPN.
 *
 * ProtonVPN is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ProtonVPN is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ProtonVPN.  If not, see <https://www.gnu.org/licenses/>.
 */

using System.ComponentModel;
using System.Threading.Tasks;
using System.Windows;
using ProtonVPN.Core;
using ProtonVPN.Windows;

namespace ProtonVPN.Login.Views
{
    public partial class LoginWindow : IOpenMainWindowAware
    {
        private readonly AppExitHandler _appExitHandler;

        public LoginWindow(AppExitHandler appExitHandler)
        {
            _appExitHandler = appExitHandler;
            InitializeComponent();

            CloseButton.Click += CloseButton_Click;
            MinimizeButton.Click += Minimize_Click;
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);
            Application.Current.Shutdown();
        }

        private void CloseButton_Click(object sender, RoutedEventArgs e)
        {
            _appExitHandler.RequestExit();
        }

        private void Minimize_Click(object sender, RoutedEventArgs e)
        {
            WindowState = WindowState.Minimized;
        }

        public async Task OnOpenMainWindow()
        {
            OpenWindow();
        }

        public void OpenWindow()
        {
            if (WindowState.Equals(WindowState.Minimized))
            {
                WindowState = WindowState.Normal;
            }
            Activate();
        }
    }
}